unit machineunit;

interface

uses
  SysUtils,
  Tomas;

Type
  TMachine = class;
  TJob = class;
  TGenerator = class;

  TMachine = class(TomasElement)
    public
    Jobs: TomasQueue;
    Job: TJob;
    Published
    Constructor Create(Name: String);
    Procedure Process; override;
    End;

  TJob = class(TomasElement)
    public
    JobTime: Double;
    End;

  TGenerator = class(TomasElement)
    public
    InterArrivals: TExponentialDistribution;
    JobTimes: TExponentialDistribution;
    Published
    Constructor Create(Name: String);
    Procedure Process; override;
    End;


implementation

Var
  Machine: TMachine;
  Generator: TGenerator;

Constructor TMachine.Create(Name: String);
Begin
  Inherited Create(Name);
  Jobs:=TomasQueue.Create('Jobs');
  Job:=Nil;
End;

Procedure TMachine.Process;
Begin
  While TRUE Do
  Begin
    While Jobs.Length > 0 Do
    Begin
      Job:=Jobs.FirstElement;
      Hold(Job.JobTime);
      Jobs.Remove(Job);
      Job.Destroy;
    End;
    Suspend;
  End;
End;

Constructor TGenerator.Create(Name: String);
Begin
  inherited Create(Name);
  InterArrivals:=TExponentialDistribution.Create(520623,30);
  JobTimes:=TExponentialDistribution.Create(1374617,24);
End;

Procedure TGenerator.Process;
var
  NewJob: TJob;
Begin
  While TRUE Do
  Begin
    NewJob:=TJob.Create('Job');
    NewJob.JobTime:=JobTimes.Sample;
    Machine.Jobs.AddToTail(NewJob);
    If Machine.Status = Passive Then
    Begin
      Machine.Resume(TNow);
    End;
    Hold(InterArrivals.Sample);
  End;
End;

Procedure InitializeJobShop;
Begin
  Machine:=TMachine.Create('Machine');
  Machine.Start(TNow);
  Generator:=TGenerator.Create('Generator');
  Generator.Start(TNow);
  TomasForm.SetTrace(TRUE);
End;

Initialization
  InitializeJobshop;

end.
